from __future__ import absolute_import

from .quick_replies import QuickReplies


class BaseAttachment(object):
    def __init__(self, attachment_type, url=None, is_reusable=None,
                 quick_replies=None, attachment_id=None):
        self.attachment_type = attachment_type
        self.url = url
        self.is_reusable = is_reusable
        self.attachment_id = attachment_id

        if quick_replies and not isinstance(quick_replies, QuickReplies):
            raise ValueError('quick_replies must be an instance of QuickReplies.')
        self.quick_replies = quick_replies

    def to_dict(self):
        d = {
            'attachment': {
                'type': self.attachment_type,
                'payload': {}
            }
        }

        if self.url:
            d['attachment']['payload']['url'] = self.url

        if self.is_reusable:
            d['attachment']['payload']['is_reusable'] = 'true'

        if self.attachment_id:
            d['attachment']['payload']['attachment_id'] = self.attachment_id

        if self.quick_replies:
            d['quick_replies'] = self.quick_replies.to_dict()

        return d


class Image(BaseAttachment):
    def __init__(self, url=None, is_reusable=None, quick_replies=None, attachment_id=None):
        self.attachment_type = 'image'
        self.url = url
        self.is_reusable = is_reusable
        self.quick_replies = quick_replies
        self.attachment_id = attachment_id
        super(Image, self).__init__(self.attachment_type, self.url, self.is_reusable,
                                    self.quick_replies, self.attachment_id)


class Audio(BaseAttachment):
    def __init__(self, url=None, is_reusable=None, quick_replies=None, attachment_id=None):
        self.attachment_type = 'audio'
        self.url = url
        self.is_reusable = is_reusable
        self.quick_replies = quick_replies
        self.attachment_id = attachment_id
        super(Audio, self).__init__(self.attachment_type, self.url, self.is_reusable,
                                    self.quick_replies, self.attachment_id)


class Video(BaseAttachment):
    def __init__(self, url=None, is_reusable=None, quick_replies=None, attachment_id=None):
        self.attachment_type = 'video'
        self.url = url
        self.is_reusable = is_reusable
        self.quick_replies = quick_replies
        self.attachment_id = attachment_id
        super(Video, self).__init__(self.attachment_type, self.url, self.is_reusable,
                                    self.quick_replies, self.attachment_id)


class File(BaseAttachment):
    def __init__(self, url=None, is_reusable=None, quick_replies=None, attachment_id=None):
        self.attachment_type = 'file'
        self.url = url
        self.is_reusable = is_reusable
        self.quick_replies = quick_replies
        self.attachment_id = attachment_id
        super(File, self).__init__(self.attachment_type, self.url, self.is_reusable,
                                   self.quick_replies, self.attachment_id)



from __future__ import absolute_import
import abc
import logging
import hashlib
import hmac
import six
import requests

__version__ = '6.0.0'

logger = logging.getLogger(__name__)


DEFAULT_API_VERSION = 2.12


class MessengerClient(object):

    # https://developers.facebook.com/docs/messenger-platform/send-messages#messaging_types
    MESSAGING_TYPES = {
        'RESPONSE',
        'UPDATE',
        'MESSAGE_TAG',
    }

    # https://developers.facebook.com/docs/messenger-platform/reference/send-api/#payload
    NOTIFICATION_TYPES = {
        'REGULAR',
        'SILENT_PUSH',
        'NO_PUSH'
    }

    def __init__(self, page_access_token, **kwargs):
        """
            @required:
                page_access_token
            @optional:
                session
                api_version
                app_secret
        """

        self.page_access_token = page_access_token
        self.session = kwargs.get('session', requests.Session())
        self.api_version = kwargs.get('api_version', DEFAULT_API_VERSION)
        self.graph_url = 'https://graph.facebook.com/v{api_version}'.format(api_version=self.api_version)
        self.app_secret = kwargs.get('app_secret')

    @property
    def auth_args(self):
        if not hasattr(self, '_auth_args'):
            auth = {
                'access_token': self.page_access_token
            }
            if self.app_secret is not None:
                appsecret_proof = self.generate_appsecret_proof()
                auth['appsecret_proof'] = appsecret_proof
            self._auth_args = auth
        return self._auth_args

    def get_user_data(self, recipient_id, fields=None, timeout=None):
        params = {}

        if isinstance(fields, six.string_types):
            params['fields'] = fields
        elif isinstance(fields, (list, tuple)):
            params['fields'] = ','.join(fields)
        else:
            params['fields'] = 'first_name,last_name,profile_pic,locale,timezone,gender'

        params.update(self.auth_args)

        r = self.session.get(
            '{graph_url}/{recipient_id}'.format(graph_url=self.graph_url, recipient_id=recipient_id),
            params=params,
            timeout=timeout
        )
        return r.json()

    def send(self, payload, recipient_id, messaging_type='RESPONSE', notification_type='REGULAR',
             timeout=None, tag=None):
        if messaging_type not in self.MESSAGING_TYPES:
            raise ValueError('`{}` is not a valid `messaging_type`'.format(messaging_type))

        if notification_type not in self.NOTIFICATION_TYPES:
            raise ValueError('`{}` is not a valid `notification_type`'.format(notification_type))

        body = {
            'messaging_type': messaging_type,
            'notification_type': notification_type,
            'recipient': {
                'id': recipient_id,
            },
            'message': payload,
        }

        if tag:
            body['tag'] = tag

        r = self.session.post(
            '{graph_url}/me/messages'.format(graph_url=self.graph_url),
            params=self.auth_args,
            json=body,
            timeout=timeout
        )
        return r.json()

    def send_action(self, sender_action, recipient_id, timeout=None):
        r = self.session.post(
            '{graph_url}/me/messages'.format(graph_url=self.graph_url),
            params=self.auth_args,
            json={
                'recipient': {
                    'id': recipient_id,
                },
                'sender_action': sender_action
            },
            timeout=timeout
        )
        return r.json()

    def subscribe_app_to_page(self, timeout=None):
        r = self.session.post(
            '{graph_url}/me/subscribed_apps'.format(graph_url=self.graph_url),
            params=self.auth_args,
            timeout=timeout
        )
        return r.json()

    def set_messenger_profile(self, data, timeout=None):
        r = self.session.post(
            '{graph_url}/me/messenger_profile'.format(graph_url=self.graph_url),
            params=self.auth_args,
            json=data,
            timeout=timeout
        )
        return r.json()

    def delete_get_started(self, timeout=None):
        r = self.session.delete(
            '{graph_url}/me/messenger_profile'.format(graph_url=self.graph_url),
            params=self.auth_args,
            json={
                'fields': [
                    'get_started'
                ],
            },
            timeout=timeout
        )
        return r.json()

    def delete_persistent_menu(self, timeout=None):
        r = self.session.delete(
            '{graph_url}/me/messenger_profile'.format(graph_url=self.graph_url),
            params=self.auth_args,
            json={
                'fields': [
                    'persistent_menu'
                ],
            },
            timeout=timeout
        )
        return r.json()

    def link_account(self, account_linking_token, timeout=None):
        r = self.session.post(
            '{graph_url}/me'.format(graph_url=self.graph_url),
            params=dict({
                'fields': 'recipient',
                'account_linking_token': account_linking_token
            }, **self.auth_args),
            timeout=timeout
        )
        return r.json()

    def unlink_account(self, psid, timeout=None):
        r = self.session.post(
            '{graph_url}/me/unlink_accounts'.format(graph_url=self.graph_url),
            params=self.auth_args,
            json={
                'psid': psid
            },
            timeout=timeout
        )
        return r.json()

    def update_whitelisted_domains(self, domains, timeout=None):
        if not isinstance(domains, list):
            domains = [domains]
        r = self.session.post(
            '{graph_url}/me/messenger_profile'.format(graph_url=self.graph_url),
            params=self.auth_args,
            json={
                'whitelisted_domains': domains
            },
            timeout=timeout
        )
        return r.json()

    def remove_whitelisted_domains(self, timeout=None):
        r = self.session.delete(
            '{graph_url}/me/messenger_profile'.format(graph_url=self.graph_url),
            params=self.auth_args,
            json={
                'fields':[
                    'whitelisted_domains'
                ],
            },
            timeout=timeout
        )
        return r.json()

    def upload_attachment(self, attachment, timeout=None):
        if not attachment.url:
            raise ValueError('Attachment must have `url` specified')
        if attachment.quick_replies:
            raise ValueError('Attachment may not have `quick_replies`')
        r = self.session.post(
            '{graph_url}/me/message_attachments'.format(graph_url=self.graph_url),
            params=self.auth_args,
            json={
                'message':  attachment.to_dict()
            },
            timeout=timeout
        )
        return r.json()

    def generate_appsecret_proof(self):
        """
            @outputs:
                appsecret_proof: HMAC-SHA256 hash of page access token
                    using app_secret as the key
        """
        app_secret = str(self.app_secret).encode('utf8')
        access_token = str(self.page_access_token).encode('utf8')

        return hmac.new(app_secret, access_token, hashlib.sha256).hexdigest()


class BaseMessenger(object):
    __metaclass__ = abc.ABCMeta

    last_message = {}

    def __init__(self, page_access_token, app_secret=None):
        self.page_access_token = page_access_token
        self.app_secret = app_secret
        self.client = MessengerClient(self.page_access_token, app_secret=self.app_secret)

    @abc.abstractmethod
    def account_linking(self, message):
        """Method to handle `account_linking`"""

    @abc.abstractmethod
    def message(self, message):
        """Method to handle `messages`"""

    @abc.abstractmethod
    def delivery(self, message):
        """Method to handle `message_deliveries`"""

    @abc.abstractmethod
    def optin(self, message):
        """Method to handle `messaging_optins`"""

    @abc.abstractmethod
    def postback(self, message):
        """Method to handle `messaging_postbacks`"""

    @abc.abstractmethod
    def read(self, message):
        """Method to handle `message_reads`"""

    def handle(self, payload):
        for entry in payload['entry']:
            for message in entry['messaging']:
                self.last_message = message
                if message.get('account_linking'):
                    return self.account_linking(message)
                elif message.get('delivery'):
                    return self.delivery(message)
                elif message.get('message'):
                    return self.message(message)
                elif message.get('optin'):
                    return self.optin(message)
                elif message.get('postback'):
                    return self.postback(message)
                elif message.get('read'):
                    return self.read(message)

    def get_user(self, fields=None, timeout=None):
        return self.client.get_user_data(self.get_user_id(), fields=fields, timeout=timeout)

    def send(self, payload, messaging_type='RESPONSE', notification_type='REGULAR', timeout=None, tag=None):
        return self.client.send(payload, self.get_user_id(), messaging_type=messaging_type,
                                notification_type=notification_type, timeout=timeout, tag=tag)

    def send_action(self, sender_action, timeout=None):
        return self.client.send_action(sender_action, self.get_user_id(), timeout=timeout)

    def get_user_id(self):
        return self.last_message['sender']['id']

    def subscribe_app_to_page(self, timeout=None):
        return self.client.subscribe_app_to_page(timeout=timeout)

    def set_messenger_profile(self, data, timeout=None):
        return self.client.set_messenger_profile(data, timeout=timeout)

    def delete_get_started(self, timeout=None):
        return self.client.delete_get_started(timeout=timeout)

    def link_account(self, account_linking_token, timeout=None):
        return self.client.link_account(account_linking_token, timeout=timeout)

    def unlink_account(self, psid, timeout=None):
        return self.client.unlink_account(psid, timeout=timeout)

    def add_whitelisted_domains(self, domains, timeout=None):
        return self.client.update_whitelisted_domains(domains, timeout=timeout)

    def remove_whitelisted_domains(self, timeout=None):
        return self.client.remove_whitelisted_domains(timeout=timeout)

    def upload_attachment(self, attachment, timeout=None):
        return self.client.upload_attachment(attachment, timeout=timeout)



class SenderAction(object):
    SENDER_ACTIONS = [
        'mark_seen',
        'typing_on',
        'typing_off'
    ]

    def __init__(self, sender_action):
        if sender_action not in self.SENDER_ACTIONS:
            raise ValueError('Invalid sender_action provided.')
        self.sender_action = sender_action

    def to_dict(self):
        return self.sender_action



from __future__ import absolute_import

import logging

from .error_messages import CHARACTER_LIMIT_MESSAGE

logger = logging.getLogger(__name__)

WEBVIEW_HEIGHT_RATIOS = [
    'compact',
    'tall',
    'full',
]


class Text(object):
    def __init__(self, text, quick_replies=None):
        self.text = text
        self.quick_replies = quick_replies

    def to_dict(self):
        d = {
            'text': self.text
        }

        if self.quick_replies:
            d['quick_replies'] = self.quick_replies.to_dict()

        return d


class DynamicText(object):
    def __init__(self, text, fallback_text=None, quick_replies=None):
        self.text = text
        self.fallback_text = fallback_text
        self.quick_replies = quick_replies

    def to_dict(self):
        dynamic_text = {
            'text': self.text,
        }

        if self.fallback_text:
            dynamic_text['fallback_text'] = self.fallback_text

        d = {
            'dynamic_text': dynamic_text,
        }

        if self.quick_replies:
            d['quick_replies'] = self.quick_replies.to_dict()

        return d


class Button(object):
    BUTTON_TYPES = [
        'web_url',
        'postback',
        'phone_number',
        'account_link',
        'account_unlink',
        'element_share',
    ]

    def __init__(self, button_type, title=None, url=None,
                 payload=None, webview_height_ratio=None,
                 messenger_extensions=None, fallback_url=None,
                 share_contents=None):

        if button_type not in self.BUTTON_TYPES:
            raise ValueError('Invalid button_type provided.')
        if webview_height_ratio and webview_height_ratio not in WEBVIEW_HEIGHT_RATIOS:
            raise ValueError('Invalid webview_height_ratio provided.')
        if title and len(title) > 20:
            logger.warning(CHARACTER_LIMIT_MESSAGE.format(field='Title',
                                                          maxsize=20))

        self.button_type = button_type
        self.title = title
        self.url = url
        self.payload = payload
        self.webview_height_ratio = webview_height_ratio
        self.messenger_extensions = messenger_extensions
        self.fallback_url = fallback_url
        self.share_contents = share_contents

    def to_dict(self):
        d = {
            'type': self.button_type,
        }

        if self.title:
            d['title'] = self.title
        if self.url:
            d['url'] = self.url
        if self.payload:
            d['payload'] = self.payload
        if self.button_type == 'web_url':
            if self.webview_height_ratio:
                d['webview_height_ratio'] = self.webview_height_ratio
            if self.messenger_extensions:
                d['messenger_extensions'] = 'true'
            if self.fallback_url:
                d['fallback_url'] = self.fallback_url
        if self.button_type == 'element_share':
            if self.share_contents:
                d['share_contents'] = self.share_contents
        return d


class Element(object):
    """
    To be used with the generic template to create a carousel
    """

    def __init__(self, title, item_url=None, image_url=None,
                 subtitle=None, buttons=None, quantity=None,
                 price=None, currency=None, default_action=None):

        self.title = title
        self.item_url = item_url
        self.image_url = image_url
        self.subtitle = subtitle
        self.buttons = buttons
        self.quantity = quantity
        self.price = price
        self.currency = currency

        if default_action:
            if default_action.title:
                raise ValueError('The default_action button may not have a title')
            if default_action.button_type != 'web_url':
                raise ValueError('The default_action button must be of type web_url')
        self.default_action = default_action

    @property
    def title(self):
        return self._title

    @title.setter
    def title(self, title):
        if len(title) > 80:
            logger.warning(CHARACTER_LIMIT_MESSAGE.format(field='Title',
                                                          maxsize=80))
        self._title = title

    @property
    def subtitle(self):
        return self._subtitle

    @subtitle.setter
    def subtitle(self, subtitle):
        if subtitle is not None and len(subtitle) > 80:
            logger.warning(CHARACTER_LIMIT_MESSAGE.format(field='Subtitle',
                                                          maxsize=80))
        self._subtitle = subtitle

    def to_dict(self):
        d = {
            'title': self.title,
        }
        if self.item_url:
            d['item_url'] = self.item_url
        if self.image_url:
            d['image_url'] = self.image_url
        if self.subtitle:
            d['subtitle'] = self.subtitle
        if self.quantity:
            d['quantity'] = self.quantity
        if self.price:
            d['price'] = self.price
        if self.currency:
            d['currency'] = self.currency
        if self.default_action:
            d['default_action'] = self.default_action.to_dict()
        if self.buttons:
            d['buttons'] = [
                button.to_dict() for button in self.buttons
            ]

        return d


class Adjustment(object):
    def __init__(self, name=None, amount=None):
        # Optional
        self.name = name
        self.amount = amount

    def to_dict(self):
        return {
            'name': self.name,
            'amount': self.amount
        }


class Address(object):
    def __init__(self, street_1, city, postal_code,
                 state, country, street_2=''):
        # Required
        self.street_1 = street_1
        self.city = city
        self.postal_code = postal_code
        self.state = state
        self.country = country
        # Optional
        self.street_2 = street_2

    def to_dict(self):
        return {
            'street_1': self.street_1,
            'street_2': self.street_2,
            'city': self.city,
            'postal_code': self.postal_code,
            'state': self.state,
            'country': self.country,
        }


class Summary(object):
    def __init__(self, total_cost, subtotal=None, shipping_cost=None, total_tax=None):
        # Required
        self.total_cost = total_cost
        # Optional
        self.subtotal = subtotal
        self.shipping_cost = shipping_cost
        self.total_tax = total_tax

    def to_dict(self):
        return {
            'subtotal': self.subtotal,
            'shipping_cost': self.shipping_cost,
            'total_tax': self.total_tax,
            'total_cost': self.total_cost
        }



from __future__ import absolute_import

from .elements import WEBVIEW_HEIGHT_RATIOS

DEFAULT_LOCALE = 'default'

class GreetingText(object):
    def __init__(self, text, locale=None):
        if len(text) > 160:
            raise ValueError('Text cannot be longer 160 characters.')
        self.text = text
        self.locale = locale or DEFAULT_LOCALE

    def to_dict(self):
        return {
            'locale': self.locale,
            'text': self.text,
            }


class GetStartedButton(object):
    def __init__(self, payload):
        self.payload = payload

    def to_dict(self):
        return {
            'payload': self.payload,
            }


class PersistentMenuItem(object):
    ITEM_TYPES = [
        'nested',
        'web_url',
        'postback'
    ]

    def __init__(self, item_type, title, nested_items=None, url=None,
                 payload=None, fallback_url=None, messenger_extensions=None,
                 webview_share_button=None, webview_height_ratio=None):
        if item_type not in self.ITEM_TYPES:
            raise ValueError('Invalid item_type provided.')
        if len(title) > 30:
            raise ValueError('Title cannot be longer 30 characters.')
        if payload and len(payload) > 1000:
            raise ValueError('Payload cannot be longer 1000 characters.')
        if item_type == 'nested':
            if not nested_items:
                raise ValueError('`nested_items` must be supplied for `nested` type menu items.')
            if len(nested_items) > 5:
                raise ValueError('Cannot have more than 5 nested_items')
        if item_type == 'web_url':
            if url is None:
                raise ValueError('`url` must be supplied for `web_url` type menu items.')
            if webview_height_ratio and not webview_height_ratio in WEBVIEW_HEIGHT_RATIOS:
                raise ValueError('Invalid webview_height_ratio provided.')
        else:
            if messenger_extensions is not None:
                raise ValueError('`messenger_extensions` is only valid for item type `web_url`')
            if webview_share_button is not None:
                raise ValueError('`webview_share_button` is only valid for item type `web_url`')
            if webview_height_ratio is not None:
                raise ValueError('`webview_height_ratio` is only valid for item type `web_url`')

        if item_type == 'postback' and payload is None:
            raise ValueError('`payload` must be supplied for `postback` type menu items.')

        self.item_type = item_type
        self.title = title
        self.nested_items = nested_items
        self.url = url
        self.fallback_url = fallback_url
        self.messenger_extensions = messenger_extensions
        self.webview_share_button = webview_share_button
        self.webview_height_ratio = webview_height_ratio
        self.payload = payload

    def to_dict(self):
        res = {
            'type': self.item_type,
            'title': self.title
        }

        if self.nested_items and self.item_type == 'nested':
            res['call_to_actions'] = [item.to_dict() for item in self.nested_items]

        if self.item_type == 'web_url':
            if self.url:
                res['url'] = self.url
            if self.fallback_url:
                res['fallback_url'] = self.fallback_url
            if self.messenger_extensions is not None:
                res['messenger_extensions'] = self.messenger_extensions
            if self.webview_share_button is False:
                res['webview_share_button'] = 'hide'
            if self.webview_height_ratio:
                res['webview_height_ratio'] = self.webview_height_ratio

        if self.payload and self.item_type == 'postback':
            res['payload'] = self.payload
        return res


class PersistentMenu(object):
    def __init__(self, menu_items=None, locale=None, composer_input_disabled=None):
        if composer_input_disabled != False:
            if not menu_items:
                raise ValueError('You must supply at least one menu_item.')

            elif len(menu_items) > 3:
                raise ValueError('You cannot have more than 3 menu_items in top level.')

        self.menu_items = menu_items
        self.locale = locale or DEFAULT_LOCALE
        self.composer_input_disabled = composer_input_disabled

    def to_dict(self):
        res = {
            'locale': self.locale,
        }

        if self.menu_items:
            res['call_to_actions'] = [
                item.to_dict() for item in self.menu_items
            ]

        if self.composer_input_disabled is not None:
            res['composer_input_disabled'] = self.composer_input_disabled

        return res


class MessengerProfile(object):
    def __init__(self, greetings=None, get_started=None, persistent_menus=None):
        self.greetings = greetings
        self.get_started = get_started
        self.persistent_menus = persistent_menus

    def to_dict(self):
        res = {}

        if self.greetings:
            res['greeting'] = [
                item.to_dict() for item in self.greetings]

        if self.get_started:
            res['get_started'] = self.get_started.to_dict()

        if self.persistent_menus:
            res['persistent_menu'] = [
                item.to_dict() for item in self.persistent_menus]

        return res



from __future__ import absolute_import

import collections
from .quick_replies import QuickReplies


class BaseTemplate(object):
    TEMPLATE_TYPE = 'base'

    def __init__(self, quick_replies=None):
        if quick_replies and not isinstance(quick_replies, QuickReplies):
            raise ValueError('quick_replies must be an instance of QuickReplies.')
        self.quick_replies = quick_replies

        self._d = {
            'attachment': {
                'type': 'template',
                'payload': {
                    'template_type': self.TEMPLATE_TYPE,
                },
            },
        }

    def to_dict(self):
        if self.quick_replies:
            self._d['quick_replies'] = self.quick_replies.to_dict()

        return self._d


class ElementMixin(object):
    MIN_ELEMENTS = 0
    MAX_ELEMENTS = 0

    def __init__(self, elements=None, *args, **kwargs):
        self.elements = elements

        super(ElementMixin, self).__init__(*args, **kwargs)

    @property
    def elements(self):
        return self._elements

    @elements.setter
    def elements(self, elements):
        if elements:
            if isinstance(elements, collections.Iterable):
                elements = list(elements)
            else:
                elements = [elements]

        self._elements = elements

    def to_dict(self):
        if self.MIN_ELEMENTS and (not self.elements or len(self.elements) < self.MIN_ELEMENTS):
            raise ValueError('At least {} elements are required.'.format(self.MIN_ELEMENTS))

        if self.elements:
            if len(self.elements) > self.MAX_ELEMENTS:
                raise ValueError('You cannot have more than {} elements in the template.'.format(self.MAX_ELEMENTS))

            self._d['attachment']['payload']['elements'] = [
                element.to_dict() for element in self.elements
            ]

        return super(ElementMixin, self).to_dict()


class ButtonMixin(object):
    MIN_BUTTONS = 0
    MAX_BUTTONS = 0

    def __init__(self, buttons=None, *args, **kwargs):
        self.buttons = buttons

        super(ButtonMixin, self).__init__(*args, **kwargs)

    @property
    def buttons(self):
        return self._buttons

    @buttons.setter
    def buttons(self, buttons):
        if buttons:
            if isinstance(buttons, collections.Iterable):
                buttons = list(buttons)
            else:
                buttons = [buttons]

        self._buttons = buttons

    def to_dict(self):
        if self.MIN_BUTTONS and (not self.buttons or len(self.buttons) < self.MIN_BUTTONS):
            raise ValueError('At least {} buttons are required.'.format(self.MIN_BUTTONS))

        if self.buttons:
            if len(self.buttons) > self.MAX_BUTTONS:
                raise ValueError('You cannot have more than {} buttons in the template.'.format(self.MAX_BUTTONS))

            self._d['attachment']['payload']['buttons'] = [
                button.to_dict() for button in self.buttons
            ]

        return super(ButtonMixin, self).to_dict()


class SharableMixin(object):
    def __init__(self, sharable=False, *args, **kwargs):
        self.sharable = bool(sharable)

        super(SharableMixin, self).__init__(*args, **kwargs)

    def to_dict(self):
        self._d['attachment']['payload']['sharable'] = self.sharable

        return super(SharableMixin, self).to_dict()


class GenericTemplate(ElementMixin, SharableMixin, BaseTemplate):
    TEMPLATE_TYPE = 'generic'

    MIN_ELEMENTS = 1
    MAX_ELEMENTS = 10

    def __init__(self, elements, quick_replies=None, image_aspect_ratio=None, **kwargs):
        self.image_aspect_ratio = image_aspect_ratio

        super(GenericTemplate, self).__init__(
            elements=elements,
            quick_replies=quick_replies,
            **kwargs
            )

    def to_dict(self):
        if self.image_aspect_ratio:
            self._d['attachment']['payload']['image_aspect_ratio'] = self.image_aspect_ratio

        return super(GenericTemplate, self).to_dict()


class MediaTemplate(BaseTemplate):
    TEMPLATE_TYPE = 'media'

    VALID_MEDIA_TYPES = ('image', 'video')

    def __init__(self, media, buttons=None):
        if media.attachment_type not in self.VALID_MEDIA_TYPES:
            raise ValueError('Only image and video types are supported')
        self.media = media
        self.buttons = buttons
        super(MediaTemplate, self).__init__()

    def to_dict(self):
        # Media's dict has an extra layer of structure we don't need
        media_dict = self.media.to_dict()
        element = media_dict['attachment']['payload']
        element['media_type'] = self.media.attachment_type
        if self.buttons:
            element['buttons'] = [b.to_dict() for b in self.buttons]
        self._d['attachment']['payload']['elements'] = [element]
        return super(MediaTemplate, self).to_dict()



class ButtonTemplate(ButtonMixin, BaseTemplate):
    TEMPLATE_TYPE = 'button'

    MIN_BUTTONS = 1
    MAX_BUTTONS = 3

    def __init__(self, text, buttons, quick_replies=None, **kwargs):
        self.text = text

        super(ButtonTemplate, self).__init__(
            buttons=buttons,
            quick_replies=quick_replies,
            **kwargs
            )

    def to_dict(self):
        self._d['attachment']['payload']['text'] = self.text
        return super(ButtonTemplate, self).to_dict()


class ListTemplate(ButtonMixin, ElementMixin, BaseTemplate):
    TEMPLATE_TYPE = 'list'

    MIN_BUTTONS = 0
    MAX_BUTTONS = 1

    MIN_ELEMENTS = 2
    MAX_ELEMENTS = 4

    def __init__(self, elements, top_element_style=None, **kwargs):
        self.top_element_style = top_element_style

        super(ListTemplate, self).__init__(elements=elements, **kwargs)

    def to_dict(self):
        if self.top_element_style:
            self._d['attachment']['payload']['top_element_style'] = self.top_element_style

        return super(ListTemplate, self).to_dict()


class ReceiptTemplate(ElementMixin, SharableMixin, BaseTemplate):
    TEMPLATE_TYPE = 'receipt'

    MIN_ELEMENTS = 0
    MAX_ELEMENTS = 100

    def __init__(self, recipient_name, order_number, currency, payment_method,
                 elements, summary, order_url=None, timestamp=None,
                 address=None, adjustments=None, quick_replies=None,
                 **kwargs):

        self.recipient_name = recipient_name
        self.order_number = order_number
        self.currency = currency
        self.payment_method = payment_method
        self.summary = summary.to_dict()
        self.order_url = order_url
        self.timestamp = timestamp
        self.address = address
        self.adjustments = adjustments
        self.quick_replies = quick_replies

        super(ReceiptTemplate, self).__init__(
            elements=elements,
            quick_replies=quick_replies,
            **kwargs
            )

    def to_dict(self):
        payload = {
            'recipient_name': self.recipient_name,
            'order_number': self.order_number,
            'order_url': self.order_url,
            'currency': self.currency,
            'timestamp': self.timestamp,
            'payment_method': self.payment_method,
            'summary': self.summary
        }

        if self.address:
            payload['address'] = self.address.to_dict()

        if self.adjustments:
            payload['adjustments'] = [
                adjustment.to_dict() for adjustment in self.adjustments
            ]

        self._d['attachment']['payload'].update(payload)

        return super(ReceiptTemplate, self).to_dict()



from __future__ import absolute_import

import logging

from .error_messages import CHARACTER_LIMIT_MESSAGE

logger = logging.getLogger(__name__)


class QuickReply(object):

    CONTENT_TYPES = [
        'text',
        'location',
        'user_phone_number',
        'user_email'
    ]

    def __init__(self, title=None, payload=None, image_url=None, content_type=None):

        if content_type is None:
            content_type = 'text'
        if content_type not in self.CONTENT_TYPES:
            raise ValueError('Invalid content_type provided.')
        if title and len(title) > 20:
            logger.warning(CHARACTER_LIMIT_MESSAGE.format(field='Title',
                                                          maxsize=20))
        if payload and len(payload) > 1000:
            raise ValueError('Payload cannot be longer 1000 characters.')

        self.title = title
        self.payload = payload
        self.image_url = image_url
        self.content_type = content_type

    def to_dict(self):
        d = {
            'content_type': self.content_type,
        }

        if self.title:
            d['title'] = self.title
        if self.payload:
            d['payload'] = self.payload
        if self.image_url:
            d['image_url'] = self.image_url

        return d


class QuickReplies(object):
    def __init__(self, quick_replies):
        if len(quick_replies) > 11:
            raise ValueError('You cannot have more than 10 quick replies.')
        self.quick_replies = quick_replies

    def __bool__(self):
        return bool(self.quick_replies)

    __nonzero__ = __bool__

    def to_dict(self):
        return [
            quick_reply.to_dict() for quick_reply in self.quick_replies
        ]



import re
import sys

from setuptools import setup
from setuptools.command.test import test as TestCommand

long_description = ''

try:
    import pandoc

    doc = pandoc.Document()
    with open('README.md', 'r') as readme:
        doc.markdown = readme.read().encode('utf-8')
        long_description = doc.rst.decode()
except Exception as e:
    print(e)
    pass

test_requirements = [
    'pytest',
    'coverage',
    'pytest-cov',
    'pytest-catchlog',
    'responses'
]

# Get the version
version_regex = r'__version__ = ["\']([^"\']*)["\']'
with open('fbmessenger/__init__.py', 'r') as f:
    text = f.read()
    match = re.search(version_regex, text)

    if match:
        VERSION = match.group(1)
    else:
        raise RuntimeError("No version number found!")


class PyTest(TestCommand):
    user_options = [('pytest-args=', 'a', "Arguments to pass into py.test")]

    def initialize_options(self):
        TestCommand.initialize_options(self)
        self.pytest_args = []

    def finalize_options(self):
        TestCommand.finalize_options(self)
        self.test_args = []
        self.test_suite = True

    def run_tests(self):
        import pytest

        errno = pytest.main(self.pytest_args)
        sys.exit(errno)


setup(
    name='fbmessenger',
    version=VERSION,
    description='A python library to communicate with the Facebook Messenger API\'s',
    long_description=long_description,
    url='https://github.com/rehabstudio/fbmessenger',
    author='Ricky Dunlop',
    author_email='hello@rickydunlop.co.uk',
    license='Apache',
    test_suite='tests',
    classifiers=[
        'Development Status :: 5 - Production/Stable',
        'Intended Audience :: Developers',
        'Natural Language :: English',
        'Topic :: Software Development :: Build Tools',
        'License :: OSI Approved :: Apache Software License',
        'Programming Language :: Python :: 2.7',
        'Programming Language :: Python :: 3',
        'Programming Language :: Python :: 3.3',
        'Programming Language :: Python :: 3.4',
        'Programming Language :: Python :: 3.5',
    ],
    install_requires=['requests>=2.0'],
    packages=['fbmessenger'],
    cmdclass={'test': PyTest},
    tests_require=test_requirements,
    keywords='Facebook Messenger Bot Chatbot',
)
